from typing import Any, Union, Text

from Crypto.PublicKey.RSA import _RSAobj

class PKCS115_Cipher:
    def __init__(self, key: _RSAobj) -> None: ...
    def can_encrypt(self) -> bool: ...
    def can_decrypt(self) -> bool: ...
    rf: Any
    def encrypt(self, message: Union[bytes, Text]) -> bytes: ...
    def decrypt(self, ct: bytes, sentinel: Any) -> bytes: ...

def new(key: _RSAobj) -> PKCS115_Cipher: ...
