from typing import Any

NO: Any
YES: Any
TRUE: Any
FALSE: Any
ON: Any
OFF: Any
N: Any
S: Any
W: Any
E: Any
NW: Any
SW: Any
NE: Any
SE: Any
NS: Any
EW: Any
NSEW: Any
CENTER: Any
NONE: Any
X: Any
Y: Any
BOTH: Any
LEFT: Any
TOP: Any
RIGHT: Any
BOTTOM: Any
RAISED: Any
SUNKEN: Any
FLAT: Any
RIDGE: Any
GROOVE: Any
SOLID: Any
HORIZONTAL: Any
VERTICAL: Any
NUMERIC: Any
CHAR: Any
WORD: Any
BASELINE: Any
INSIDE: Any
OUTSIDE: Any
SEL: Any
SEL_FIRST: Any
SEL_LAST: Any
END: Any
INSERT: Any
CURRENT: Any
ANCHOR: Any
ALL: Any
NORMAL: Any
DISABLED: Any
ACTIVE: Any
HIDDEN: Any
CASCADE: Any
CHECKBUTTON: Any
COMMAND: Any
RADIOBUTTON: Any
SEPARATOR: Any
SINGLE: Any
BROWSE: Any
MULTIPLE: Any
EXTENDED: Any
DOTBOX: Any
UNDERLINE: Any
PIESLICE: Any
CHORD: Any
ARC: Any
FIRST: Any
LAST: Any
BUTT: Any
PROJECTING: Any
ROUND: Any
BEVEL: Any
MITER: Any
MOVETO: Any
SCROLL: Any
UNITS: Any
PAGES: Any
